"use strict";
const testrail_js_api_1 = require("testrail-js-api");
const config_1 = require("./config");
const upload_screenshots_1 = require("./utils/upload-screenshots");
const upload_videos_1 = require("./utils/upload-videos");
const close_runs_1 = require("./utils/close-runs");
const testrail_getResults_1 = require("./utils/testrail-getResults");
const misc_1 = require("./utils/misc");
const Status = {
    Passed: {
        value: testrail_js_api_1.TestStatus.Passed,
        text: "PASSED",
        color: "yellow",
    },
    Blocked: {
        value: testrail_js_api_1.TestStatus.Blocked,
        text: "SKIPPED",
        color: "green",
    },
    Failed: {
        value: testrail_js_api_1.TestStatus.Failed,
        text: "FAILED",
        color: "red",
    },
};
const prepareRun = async (testrailAPI, config, runName, refs, caseIdList) => {
    var _a;
    const { projectId, suiteId, runDescription, runId, updateRunTestCases } = config;
    let existingRun;
    if (runId) {
        const { value: returnedRun } = await (0, misc_1.throwOnApiError)(testrailAPI.getRun(runId));
        existingRun = returnedRun;
    }
    else {
        const { value: runsResult } = await (0, misc_1.throwOnApiError)(testrailAPI.getRuns(projectId, { is_completed: 0 }));
        existingRun = (_a = runsResult === null || runsResult === void 0 ? void 0 : runsResult.runs) === null || _a === void 0 ? void 0 : _a.find((run) => run.refs === refs);
    }
    if (!updateRunTestCases) {
        if (existingRun) {
            return existingRun;
        }
        else {
            throw new Error(`[TestRail] Flag 'updateRunTestCases' enabled but the run was not found, please create it`);
        }
    }
    else if (existingRun) {
        const tests = await (0, testrail_getResults_1.getAllTests)(testrailAPI, existingRun.id);
        const currentCaseIds = (tests === null || tests === void 0 ? void 0 : tests.map((test) => test.case_id)) || [];
        const additionalDescription = "\n" + runDescription;
        const newDescription = existingRun.description
            ? existingRun.description.replace(additionalDescription, "") +
                additionalDescription
            : additionalDescription;
        await (0, misc_1.throwOnApiError)(testrailAPI.updateRun(existingRun.id, {
            description: newDescription,
            case_ids: [...currentCaseIds, ...caseIdList],
        }));
        console.log(`[TestRail] Test run updated successfully: ${runName}`);
        return existingRun;
    }
    else {
        const payload = {
            suite_id: suiteId,
            include_all: false,
            case_ids: caseIdList,
            name: runName,
            description: runDescription,
            refs,
        };
        const { value: newRun } = await (0, misc_1.throwOnApiError)(testrailAPI.addRun(projectId, payload));
        console.log(`[TestRail] Test run added successfully: ${runName}`);
        return newRun;
    }
};
class TestcafeTestrailReporter {
    constructor() {
        this.reportTaskStart = async (_startTime, userAgents) => {
            this.userAgents = userAgents;
        };
        this.reportFixtureStart = async () => {
        };
        this.reportTestDone = async (name, testRunInfo, meta, formatError) => {
            var _a;
            const hasErr = testRunInfo.errs.length;
            let testStatus = null;
            if (testRunInfo.skipped) {
                testStatus = Status.Blocked;
            }
            else if (hasErr === 0) {
                testStatus = Status.Passed;
            }
            else {
                testStatus = Status.Failed;
            }
            const caseIdList = [];
            if (meta[this.config.caseMeta]) {
                if (typeof meta[this.config.caseMeta] === "string") {
                    caseIdList.push(parseInt(meta[this.config.caseMeta].replace("C", "").trim(), 10));
                }
                else {
                    for (let i = 0; i < meta[this.config.caseMeta].length; i++) {
                        caseIdList.push(Number(meta[this.config.caseMeta][i].replace("C", "").trim()));
                    }
                }
            }
            if (caseIdList.length > 0) {
                for (let i = 0; i < caseIdList.length; i++) {
                    const errorLog = testRunInfo.errs
                        .map((x) => {
                        const formatted = formatError(x).replace(/[\u001b\u009b][[()#;?]*(?:[0-9]{1,4}(?:;[0-9]{0,4})*)?[0-9A-ORZcf-nqry=><]/g, "");
                        return formatted;
                    })
                        .join("\n");
                    let testDuration;
                    if (testRunInfo.durationMs > 0) {
                        testDuration = (testRunInfo.durationMs / 1000).toString() + "s";
                    }
                    this.results.push({
                        case_id: caseIdList[i],
                        status_id: testStatus.value,
                        comment: `Test ${testStatus.text}\n${errorLog}`,
                        elapsed: testDuration,
                    });
                    if (testRunInfo.screenshots.length) {
                        this.screenshots[caseIdList[i]] = testRunInfo.screenshots;
                    }
                    if ((_a = testRunInfo.videos) === null || _a === void 0 ? void 0 : _a.length) {
                        this.videos[caseIdList[i]] = testRunInfo.videos;
                    }
                }
            }
            else {
                console.log(`[TestRail] Test missing the TestRail Case ID in test metadata: ${name}`);
            }
        };
        this.reportTaskDone = async (_endTime, _passed, _warnings, _result) => {
            const { host, user, apiKey } = this.config;
            if ((0, config_1.verifyConfig)(this.config)) {
                try {
                    if (this.results.length) {
                        const runName = (0, misc_1.prepareReportName)(this.config, this.branch, this.buildNo, this.userAgents);
                        const refs = (0, misc_1.prepareReference)(this.config, this.branch, this.buildNo);
                        const caseIdList = this.results.map((result) => result.case_id);
                        const testrailAPI = new testrail_js_api_1.TestRail(host, user, apiKey);
                        const cases = await (0, testrail_getResults_1.getAllCases)(testrailAPI, this.config);
                        const existingCaseIds = cases.map((item) => item.id);
                        caseIdList.forEach((id) => {
                            if (!existingCaseIds.includes(id)) {
                                console.error(`[TestRail] All TestRail mappings should be valid. Following test case id does not exist in TestRail: ${id}.`);
                            }
                        });
                        const run = await prepareRun(testrailAPI, this.config, runName, refs, caseIdList);
                        await this.publishTestResults(testrailAPI, run, this.results);
                        await (0, close_runs_1.closeOldRuns)(testrailAPI, this.config);
                    }
                    else {
                        console.log("[TestRail] No test case data found to publish");
                    }
                }
                catch (error) {
                    console.error("[TestRail] Sending report to TestRail failed", error);
                    throw error;
                }
            }
        };
        this.publishTestResults = async (testrailAPI, run, resultsToPush) => {
            const runId = run.id;
            const { value: results } = await (0, misc_1.throwOnApiError)(testrailAPI.addResultsForCases(runId, resultsToPush));
            const tests = await (0, testrail_getResults_1.getAllTests)(testrailAPI, runId);
            if (this.config.uploadScreenshots) {
                await (0, upload_screenshots_1.uploadScreenshots)({
                    tests,
                    results,
                    resultsToPush,
                    screenshots: this.screenshots,
                    testrailAPI,
                });
            }
            if (this.config.uploadVideos) {
                await (0, upload_videos_1.uploadVideos)({
                    tests,
                    results,
                    resultsToPush,
                    videos: this.videos,
                    testrailAPI,
                });
            }
            if (results.length == 0) {
                console.log("[TestRail] No data has been published to TestRail");
            }
            else {
                console.log("[TestRail] Test results added to the TestRail successfully");
            }
        };
        this.config = (0, config_1.prepareConfig)();
        this.branch = process.env[this.config.branchEnv] || "master";
        this.buildNo = process.env[this.config.buildNoEnv] || "unknown";
        this.noColors = false;
        this.results = [];
        this.screenshots = {};
        this.videos = {};
    }
}
module.exports = () => {
    const reporter = new TestcafeTestrailReporter();
    return {
        reportTaskStart: reporter.reportTaskStart,
        reportFixtureStart: reporter.reportFixtureStart,
        async reportTestDone(name, testRunInfo, meta) {
            return reporter.reportTestDone(name, testRunInfo, meta, this.formatError.bind(this));
        },
        reportTaskDone: reporter.reportTaskDone,
        reporter,
    };
};
